; Title - Demo 256k
; By    - Michael Garber
;		- 1.00 - September 1, 2011
; About this Program
; ------------------ 
; first test of making a bank-switched demo.  This displays a 14-frame animation.

INCLUDE "HVGLIB.H"			; Home Video Game Library Header

TSVertLines		EQU	94		; Title Screen number of rows used for screen data
TSTopBorder		EQU	8		; Title Screen number of lines at top of screen to balance bottom blanking

GameVertLines		EQU	90		; # vertical lines for game

VBLPerFrame		EQU	4

; variables
CurrentIntLine	EQU $4f7a		; current interrupt line number
VBlankHit		EQU	$4f7b		; flag for hitting vblank
FrameNo			EQU	$4f7c		; sound effect volume (voice 1 & 2)
Governor		EQU	$4f7d		; sound effect volume (voice 3)
DebTrigs		EQU	$4f7e		; Controller 1-4 D-trigger (debounced)
TickRND			EQU	$4f7f		; Random byte per game tick

; leave $20 bytes for stack
StartStack		EQU	$4FA0

	ORG	FIRSTC			; First byte of Cartridge
	DB	$55				; User Cartridge Sentinel

; Menu Selection Choice #1
	DW	MENUST			; Next Menu Link
	DW	ProgramName		; Address of Program Name text
	DW	TestStart		; Jump here if selected

; Program Start
TestStart:
    LD	SP,StartStack  	; set SP lower to avoid conflict with bios scratchpad

	SYSTEM (INTPC)	   	; Start System Interpreter 
	DO	(SETOUT)	   	; Set Display Ports
	DB	(TSVertLines*2) ; Vertical Blanking Line
	DB	44			   	; Left/Right Color Boundary 
	DB	$08			   	; Interrupt Mode 
	DO	(COLSET)	   	; Set Color Registers
	DW	TitleColors	   	; Color Table

	DO	(FILL)		   	; Screen Fill at top (bg color to match blanking)
	DW	NORMEM		   	; Destination
	DW	(TSTopBorder*40)	; Bytes to move
	DB	00000000B	   	; Fill with

	; Clear screen
	DO	(FILL)			; Screen Fill
	DW	NORMEM+(TSTopBorder*40)		; Destination
	DW	((TSVertLines-TSTopBorder)*40)	; Bytes to move
	DB	01010101B		; Fill with

	DO	(STRDIS)		; Display String
	DB	28				; X
	DB	20				; Y
	DB	00001101B		; options
	DW	ProgramName		; Game Title string

	DO	(STRDIS)		; Display String
	DB	32				; X
	DB	30				; Y
	DB	00001001B		; options
	DW	VersionNo		; Version No. string

	DO	(STRDIS)		; Display String
	DB	28				; X
	DB	48				; Y
	DB	00001001B		; options
	DW	ProgrammedBy	; 

	DO	(STRDIS)		; Display String
	DB	24				; X
	DB	56				; Y
	DB	00001001B		; options
	DW	MichaelGarber	; 

	DO	(STRDIS)		; Display String
	DB	24				; X
	DB	68				; Y
	DB	00001001B		; options
	DW	RiffRaffGames	; RiffRaff Games

	EXIT				; Exit System Interpreter
	EI					; Enable interrupts 

	XOR	A
	LD	(DebTrigs),A

SplashScreen:
	CALL DebounceTriggers

SplashLoop:
	XOR	A
	SYSTEM	RANGED
	XOR	A
	CALL TestDTrigger
	JR	Z, SplashScreen	; keep showing Splash Screen unless trigger pressed
	CALL ClearDTrigger

; done with splash screen
SplashDone:

; show riff raff screen and scroll it by one pixel.
	XOR	A
	LD	(FrameNo),A
RRWait:
	CALL DebounceTriggers

	LD	A,(FrameNo)
	AND	3
	LD	L,A
	LD	H,0
	LD	BC,$3FC0+46	; 46, 47, 48, 49 = rr frames (pre-shifted)
	ADD	HL,BC
	LD	A,(HL)  ; Switch to Bank 46+frame

	XOR	A
	LD	A,(FrameNo);
	AND	11111100B
	RRCA
	RRCA
	CALL	$3000		; draw new frame of riffraff

	LD	A,(FrameNo)
	INC	A
	CP	160
	JR	NZ,RRScroll
	XOR	A				; restart at 0
RRScroll:
	LD	(FrameNo),A

	XOR	A
	CALL TestDTrigger
	JR	Z,RRWait		; keep showing Riff Raff screen until trigger pressed
	CALL ClearDTrigger

; setup colors for pokimon movie
	DI					; Disable interrupts
	SYSTEM (INTPC)		; Start System Interpreter 
	DO	(COLSET)		; Set Color Registers for Warlords
	DW	GameColors	; Color Table
	DO	(SETOUT)	   	; Set Display Ports
	DB	(GameVertLines*2) ; Vertical Blanking Line
	DB	44 ;20			   	; Left/Right Color Boundary 
	DB	$08			   	; Interrupt Mode 
	EXIT				; Exit System Interpreter
	EI					; Enable interrupts

	CALL ClearScreen
	CALL InitScreenInt

	XOR	A
	LD	(FrameNo),A

	LD	A,VBLPerFrame
	LD	(Governor),A
; --------------------------
; ----- MAIN GAME LOOP -----
; --------------------------

GameLoop:
	XOR	A
	SYSTEM	RANGED
	LD	(TickRND),A		; grab random number

WaitForVB:
	LD	A,(VBlankHit)
	AND	00000001B
	JR	Z,WaitForVB
	XOR	A
	LD	(VBlankHit),A

	CALL DebounceTriggers

	LD	A,(Governor)
	DEC	A
	LD	(Governor),A
	JR	NZ,WaitForVB
	LD	A,VBLPerFrame
	LD	(Governor),A

	LD	A,(FrameNo)
	INC	A
	CP	14		; pokimon movie has 14 frames
	JR	C,NotOver
	XOR	A		; start over at the beginning of animation
NotOver:
	LD	(FrameNo),A
	
	LD	L,A
	LD	H,0
	LD	BC,$3FC0+$20
	ADD	HL,BC
	LD	A,(HL)  ; Switch to Bank 32+frame

	XOR	A
	CALL	$3000		; draw new frame

	JR	GameLoop
; ----------------------------------------------------------------
ClearScreen:
	SYSSUK (FILL)			; Screen Fill
	DW	NORMEM				; Destination
	DW	(GameVertLines*40)	; Bytes to move
	DB	00000000B			; Fill with
	RET
; ----------------------------------------------------------------
; clear debounce trigger (a = controller #)
ClearDTrigger:
	LD	HL, DebTrigs
	CP	0
	JR	NZ,CTNotZero
	RES	0,(HL)
	RET
CTNotZero:
	CP	1
	JR	NZ,CTNotOne
	RES	1,(HL)
	RET
CTNotOne:
	CP	2
	JR	NZ,CTNotTwo
	RES	2,(HL)
	RET
CTNotTwo:
	RES	3,(HL)
	RET
; ----------------------------------------------------------------
; test debounce trigger (a = controller #)
TestDTrigger:
	LD	HL, DebTrigs
	CP	0
	JR	NZ,TDNotZero
	BIT	0,(HL)
	RET
TDNotZero:
	CP	1
	JR	NZ,TDNotOne
	BIT	1,(HL)
	RET
TDNotOne:
	CP	2
	JR	NZ,TDNotTwo
	BIT	2,(HL)
	RET
TDNotTwo:
   	BIT	3,(HL)
	RET
; ----------------------------------------------------------------
; read joystick triggers, set proper bits in DebTrig
DebounceTriggers:
	LD	HL, DebTrigs
	IN	A,(SW0)			; Get joystick info from controller 1
	BIT	CHTRIG, A	 	; test trigger
	JR	NZ, DT1Set
; trigger not currently held down
	RES	4,(HL)
	RES	0,(HL)
	JR	DT1Cont
; trigger held down
DT1Set:
	BIT 4,(HL)			; waiting to clear bit?
	JR	NZ,DT1Cont
	SET	4,(HL)
	SET	0,(HL)
DT1Cont:
	RET
; ----------------------------------------------------------------
AnyTrigger:
	IN	A,(SW0)			; Get joystick info from controller 1
	LD	B,A
	IN	A,(SW1)			; Get joystick info from controller 2
	OR	B
	LD	B,A
	IN	A,(SW2)			; Get joystick info from controller 3
	OR	B
	LD	B,A
	IN	A,(SW3)			; Get joystick info from controller 4
	OR	B
	RET
; ----------------------------------------------------------------
ScreenBreaks:
	DB	178		; bottom of screen was 186
	DB	6		;
	DB	100		;
; ----------------------------------------------------------------
CastleBands:
	DB	$00
	DB	$01
	DB	$01
; ----------------------------------------------------------------
; init screen interrupt
InitScreenInt:
    DI
	IM2

	LD A,SIVEC>>8		; set addy of vector for int
	LD I,A
	LD A,SIVEC & $ff
	OUT ($0D), A

	LD	A,(ScreenBreaks)
	OUT ($0F), A
	XOR	A
	LD	(CurrentIntLine),A
	EI
	RET
; ----------------------------------------------------------------
; Screen interrupt
SIVEC	DW	ScreenInt
ScreenInt:
	PUSH AF
	PUSH BC
	PUSH HL
	PUSH IX

	LD	B,24
DELAYLOOP:
	DJNZ DELAYLOOP

	LD	A,(CurrentIntLine)
	LD	C,A
	LD	B,0
	LD	HL,CastleBands
	ADD	HL,BC
	LD	A,(HL)	; get color value for left color band
	OUT  (COL0L),A		; Left Color Register 0

	LD	A,(CurrentIntLine)
	CP	0
	JR	NZ,SKP1
; first line in table, placed at bottom of screen in vblank area
	LD	A,1
	LD (VBlankHit),A	; signal "vblank" to sync main game code

SKP1:
	; get to next line
	LD	A,(CurrentIntLine)
	INC A
	CP	3
	JR	C,SKP2
	XOR	A
SKP2:
	LD	(CurrentIntLine),A
	LD	C,A
	LD	B,0
	LD	HL,ScreenBreaks
	ADD	HL,BC
	LD	A,(HL)			; get next line to interrupt on
	OUT (INLIN),A		; ($0f)

SKP3:
	POP	IX
	POP	HL
	POP	BC
	POP AF
	EI
	RET
; ----------------------------------------------------------------
; Color Table #1 (grey/title screens)
TitleColors:
	DB     $07        ; Color 3 Left 
	DB     $07        ; Color 2 Left
	DB     $03        ; Color 1 Left 
	DB     $fe        ; Color 0 Left
	DB     $07        ; Color 3 Right
	DB     $07        ; Color 2 Right 
	DB     $03        ; Color 1 Right 
	DB     $fe        ; Color 0 Right

; Color Table Game
GameColors:
	DB     $73        ; Color 3 Left (brown)
	DB     $6f        ; Color 2 Left (tan)
	DB     $76        ; Color 1 Left (orange/yellow)
	DB     $01        ; Color 0 Left
	DB     $07        ; Color 3 Right
	DB     $05        ; Color 2 Right 
	DB     $03        ; Color 1 Right
	DB     $01        ; Color 0 Right

; ----------------------------------------------------------------

;INCLUDE "rr2.H"			; riffraff graphic

; Program Name
ProgramName:	DB	'256K GFX DEMO',$00
VersionNo:		DB	'VERSION 1.20',$00
RiffRaffGames	DB	'RIFFRAFF GAMES',$00
ProgrammedBy:	DB	'PROGRAMMED BY',$00
MichaelGarber:	DB	'MICHAEL GARBER',$00

	ORG $2fff
	DB 0
; End of Program
